#include <SPI.h>

#include <Wire.h>

#include <Adafruit_GFX.h>

#include <Adafruit_SSD1306.h>

#include "HalfStepper.h"

#include <YetAnotherPcInt.h>

#include <Mouse.h>

#define ARRAY_SIZE(X) (sizeof(X)/sizeof(*X))

#define SCREEN_WIDTH 128 // OLED display width, in pixels

#define SCREEN_HEIGHT 32 // OLED display height, in pixels

#define PIN_A0 16
#define PIN_A1 14
#define PIN_A2 15
#define PIN_B 8

int last_pwm_value = 127;
int pwm_value = 127;
bool last_a0 = true;
bool last_a1 = true;
bool last_a2 = true;

bool cur_a0 = false; 
bool cur_a1 = false; 
bool cur_a2 = false;

int int_2_cnt = 0;
int int_3_cnt = 0;
int int_4_cnt = 0;

bool sweep_mode;
const int sweep_range = 12;
int sweep_cnt = 0;

struct Pin {
  int number;
  const char* name;
};

Pin monitored_pins[] = {
  {14,  "D14"},
  {16,  "D16"},
};

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)

#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define STEPS 32767
Stepper stepper(STEPS, 5, 6, 9, 10);


void pinChanged(Pin* pin, bool pinstate) 
{
//pin->name
//pin->number
// pinstate 

  if (pin->number == 16)
  {
    int_2_cnt++;
  }
  else
  {
    int_3_cnt++;
  }
  int_a();
}

void setup()

{

   pinMode(PIN_A0, INPUT_PULLUP);
   pinMode(PIN_A1, INPUT_PULLUP);
   pinMode(PIN_A2, INPUT_PULLUP);
   pinMode(PIN_B, INPUT_PULLUP);

   stepper.setSpeed(10);
//   stepper.stop();

  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3C for 128x32

    Serial.println(F("SSD1306 allocation failed"));

    for(;;); // Don't proceed, loop forever

  }
  display.clearDisplay();

  display.setTextSize(2);      // Normal 1:1 pixel scale
  display.setTextColor(SSD1306_WHITE); // Draw white text
  display.cp437(true);         // Use full 256 char 'Code Page 437' font
  display.display();


for (int i=0; i<ARRAY_SIZE(monitored_pins); i++) {
    pinMode(monitored_pins[i].number, INPUT_PULLUP);
    PcInt::attachInterrupt(monitored_pins[i].number, pinChanged, &monitored_pins[i], CHANGE);
  }

 Mouse.begin();
}




void loop()

{
  bool sweep_button =  digitalRead(PIN_B);
  if (sweep_mode && sweep_button)
  {
      sweep_mode = false;
  } 
  else
  if (!sweep_mode && !sweep_button)
  {
     sweep_cnt = 0;
     sweep_mode = true;
  }
  if (sweep_mode)
  {
    stepper.step(1);
    pwm_value++;
    last_pwm_value++;
    delay(1000);
    Mouse.click();
    delay(4000);
  }
 

  display.clearDisplay();
  display.setCursor(0, 0);     // Start at top-left corner
  display.print("Target"); 
  display.setCursor(80, 0);     // Start at top-left corner
  display.print(pwm_value); 

  display.setCursor(0, 16);     // Start at top-left corner
  display.print("Actual"); 
  display.setCursor(80, 16);     // Start at top-left corner
  display.print(last_pwm_value); 


/*
  display.setCursor(0, 16);     // Start at top-left corner
  display.print(cur_a0); 
  display.setCursor(40, 16);     // Start at top-left corner
  display.print(cur_a1); 
  display.setCursor(80, 16);     // Start at top-left corner
  display.print(cur_a2); 
*/

/*
  display.setCursor(0, 16);     // Start at top-left corner
  display.print(int_2_cnt); 
//  display.print(digitalPinToInterrupt(16));
  display.setCursor(40, 16);     // Start at top-left corner
//  display.print(digitalPinToInterrupt(14));

//  display.print(digitalPinToInterrupt(PIN_A0));

  
  display.print(int_3_cnt); 
  display.setCursor(80, 16);     // Start at top-left corner
  display.print(cur_a2); 
*/    
  display.display();

// Increment PWM value and return to 0 after 255

//value=(value+1)%256;

if (!sweep_mode)
{
    if (pwm_value > last_pwm_value)
    {
        last_pwm_value++;
        stepper.step(1);

    } 
    else if (pwm_value < last_pwm_value)
    {
        last_pwm_value--;
        stepper.step(-1);
    }
}

// Small 10ms delay

//delay(10);



}



void int_a()
{
//  noInterrupts();
  
   cur_a0 = digitalRead(PIN_A0); 
   cur_a1 = digitalRead(PIN_A1); 
   cur_a2 = digitalRead(PIN_A2);

  int code = 0;
  if (last_a1) 
  {
    code |= 0x08;
  }
  if (last_a0) 
  {
    code |= 0x04;
  }
  if (cur_a1) 
  {
    code |= 0x02;
  }
  if (cur_a0) 
  {
    code |= 0x01;
  }
  
  if (code == 0x07)//(code == 0x01 || code == 0x07 || code == 0x0E || code == 0x80)
  {
    if (cur_a2)
    {
      if (pwm_value < 254)
      {
        pwm_value++;
      }
    }
    else
    {
      if (pwm_value < 244)
      {
        pwm_value+=10;
      }
    }
  } 
  else if (code == 0x0B)//(code == 0x02 || code == 0x0B || code == 0x0D || code == 0x04)
  {
    
    if (cur_a2)
    {
      if (pwm_value > 1)
      {
        pwm_value--;
      }
    }
    else
    {
      if (pwm_value > 11)
      {
        pwm_value-=10;
      }
    }
 }
  last_a0 = cur_a0;
  last_a1 = cur_a1;
  last_a2 = cur_a2;


  sweep_mode = false; 
// interrupts();
}
